<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xalan="http://xml.apache.org/xalan"
                   exclude-result-prefixes="xalan" xmlns:nk="http://1060.org">
	<xsl:output method="xml" omit-xml-declaration="yes"/>
   <xsl:variable name="explorer">/tools/explore/</xsl:variable>
   <xsl:variable name="resolver">/tools/search/resolver?resolverID=searchDocResolver&amp;documentID=</xsl:variable>
   
   	<xsl:param name="hash" nk:class="java.lang.String"/>
	<xsl:param name="space" nk:class="java.lang.String"/>
	<xsl:param name="version" nk:class="java.lang.String"/>
	<xsl:param name="type" nk:class="java.lang.String"/>
	<xsl:param name="id" nk:class="java.lang.String"/>
			 
	<xsl:template match="/meta">
		<div class="metabox">
			<style type="text/css"><![CDATA[
			.border-table { border-spacing: 0px; border: solid 1px #444; margin: 1px; }
			.border-table td { vertical-align: top; border: solid 1px #444;   }
			.table-title { background: #444;  font-weight: bold;  }
			.table-title td { vertical-align: top; border: solid 1px #444; color: white; text-align: center;}
			]]></style>
			<h1><xsl:value-of select="name"/></h1>
			<p><xsl:value-of select="desc"/></p>
			
			<div style="float: right; border: solid 1px black; margin: 4px; background: white; padding: 2px; text-align: center;">
				<a href="/tools/explore/{$hash}">
					<div><img border="0" src="/tools/introspect/pub/SpaceExplorer.png"/></div>
					<div>Space Explorer View</div>
				</a>
			</div>
			
			<xsl:choose>
				<xsl:when test="$type='overlay' or $type='transparent overlay'">
					<xsl:call-template name="overlay"/>
				</xsl:when>
				<xsl:when test="$type='transreptor' ">
					<xsl:call-template name="transreptor"/>
				</xsl:when>
				<xsl:when test="$type='accessor' ">
					<xsl:call-template name="accessor"/>
				</xsl:when>
				<xsl:when test="$type='accessor prototype' ">
					<xsl:call-template name="proto-accessor"/>
				</xsl:when>
				<xsl:when test="$type='transport' ">
					<xsl:call-template name="transport"/>
				</xsl:when>
				<xsl:when test="$type='representation' ">
					<xsl:call-template name="representation"/>
				</xsl:when>
			</xsl:choose>
				
			
			<xsl:call-template name="import"/>
			
		</div>	
	</xsl:template>
	
	<xsl:template name="import">
		<div>
			<xsl:if test="string-length($space) &gt; 0">
				<h2>Import Requirements</h2>
				<p>To use <xsl:value-of select="name"/> you must import the module
				<b><xsl:value-of select="$space"/></b>
				{xml}
				<import>
					<uri><xsl:value-of select="$space"/></uri>
				</import>
				{/xml}
				</p>
			</xsl:if>
		</div>
	</xsl:template>
	
	<xsl:template name="overlay">
		<h2>Overlays</h2>
		<p>The <xsl:value-of select="name"/> is a <a href="{$resolver}doc:glossary:overlay"><xsl:value-of select="$type"/></a>. Within your standard module an overlay must be instantiated
		from the prototype. This will create a new instance of the <xsl:value-of select="name"/> within your application space.
		Prototypes have a wrapped space which will receive all sub-requests issued by the overlay.
		<xsl:if test="$type='transparent overlay'">Because <xsl:value-of select="name"/> is a transparent overlay it will transparently
		expose all functionality from the wrapped space.</xsl:if>
		</p>
		<xsl:call-template name="proto-instantiate-example"/>

		<xsl:call-template name="parameters"/>
		
	</xsl:template>
	
	<xsl:template name="transreptor">
		<h2>Transreptors</h2>
		<p>The <xsl:value-of select="name"/> is a <a href="{$resolver}doc:glossary:transreptor">transreptor</a>. A transreptor is a component which can adapt representations
		from one type to another.</p>
		
		<p>The following table shows the transreptions that can be performed:
		<table class="border-table">
			<tr class="table-title">
				<td>from</td><td>to</td>
			</tr>
			<tr>
				<td>
					<xsl:for-each select="interface/args/arg[name='primary']/reps/rep">
						<div><xsl:call-template name="rep"/></div>
					</xsl:for-each>
				</td>
				<td>
					<xsl:for-each select="interface/reps/rep">
						<div><xsl:call-template name="rep"/></div>
					</xsl:for-each>
				</td>
			</tr>
		</table></p>
			
		
	</xsl:template>
	
	<xsl:template name="rep">
		<xsl:choose>
			<xsl:when test="name"><a href="{$explorer}{hash}"><xsl:value-of select="name"/></a></xsl:when>
			<xsl:otherwise><xsl:value-of select="class"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="accessor">
		<script language="Javascript">
		<![CDATA[
			function showGrammar()
			{	if (document.getElementById( 'grammar' ).style.display=="none")
				{	document.getElementById( 'grammar' ).style.display= "";
				}
				else
				{	document.getElementById( 'grammar' ).style.display= "none";
				}
			}			
		]]>
		</script>
		<h2>Accessors</h2>
		<p><xsl:value-of select="name"/> is an <a href="{$resolver}doc:logicalreference:module:standard:logical:accessor">accessor</a>.
		
		<p>
		<xsl:value-of select="name"/> supports the following verb<xsl:if test="count(interface/verbs/verb) > 1"><xsl:text>s</xsl:text></xsl:if>:
		<table class="border-table">
			<tr class="table-title">
				<td colspan="8">verb<xsl:if test="count(interface/verbs/verb) > 1"><xsl:text>s</xsl:text></xsl:if></td>
			</tr>
			<tr>
			<xsl:for-each select="interface/verbs/verb">
				<td>
					<a href="#{text()}"><xsl:value-of select="."/></a>
				</td>
			</xsl:for-each>
			</tr>
		</table>
		</p>
		
		<xsl:if test="interface/grammar/group[group[1]/@name='scheme' and interleave]">
			<p><xsl:value-of select="name"/> supports the active URI syntax with the following arguments:
			<table class="border-table">
				<tr class="table-title">
					<td>argument</td><td>description</td><td>inclusion</td>
				</tr>
				<xsl:for-each select="interface/grammar/group/interleave/group[count(group)=1 and group/regex/@type='active-escaped-uri-loose']">
					<tr>
						<xsl:variable name="name" select="group/@name"/>
						<td><xsl:value-of select="$name"/></td>
						<td>
							<div>
							<xsl:choose>
								<xsl:when test="/meta/interface/args/arg[name=$name]/desc"><xsl:value-of select="/meta/interface/args/arg[name=$name]/desc"/></xsl:when>
								<xsl:otherwise>[no description available]</xsl:otherwise>
							</xsl:choose>
							</div>
            </td>
            <td>
							<div>
							<xsl:choose>
								<xsl:when test="(@min='1' or not(@min)) and (@max='1' or not(@max))">Mandatory</xsl:when>
								<xsl:when test="@min='0' and (@max='1' or not(@max))">Optional</xsl:when>
								<xsl:when test="@min='0' and @max&gt;'32768'">May be specified any number of times</xsl:when>
								<xsl:otherwise>Must be specified between <xsl:value-of select="min"/> and
									<xsl:choose>
										<xsl:when test="@max&gt;'32768'">infinity</xsl:when>
										<xsl:otherwise><xsl:value-of select="@max"/></xsl:otherwise>
									</xsl:choose>
									times
								</xsl:otherwise>
							</xsl:choose>
							</div>
						</td>
					</tr>
				</xsl:for-each>
				<xsl:if test="interface/grammar//group[group[1]/@name='argName' and group[2]/@name='argValue']">
					<tr>
						<td>...</td>
						<td>This accessor supports additional arbitrary arguments.
						</td>
					</tr>
				</xsl:if>
			</table></p>
		</xsl:if>
				
		It has a <a href="{$resolver}doc:logicalreference:module:standard:logical:grammar">grammar</a>
		which can been seen
		<a href="javascript:showGrammar()">here</a>.
		</p>
		<div id="grammar" style="display:none;">
		{xml}
		<xsl:copy-of select="//grammar"/>
		{/xml}
		</div>
		
		
		<p>The <xsl:value-of select="name"/> has the logical identifier, <b><xsl:value-of select="$id"/></b>, which may be used to
		directly access this endpoint by constructing requests using NKF or any
		technology that supports the <a href="{$resolver}doc:logicalreference:module:standard:logical:declarative:request:syntax">declarative request</a>.</p>
		
		
			
	</xsl:template>
	
	<xsl:template name="proto-accessor">
		<h2>Prototype Accessors</h2>
		<p>The <xsl:value-of select="name"/> is an <a href="{$resolver}doc:logicalreference:module:standard:logical:accessor">accessor</a> that must be instantiated from a
		<a href="{$resolver}doc:logicalreference:module:standard:logical:prototype">prototype</a>. When instantiated the prototype will create an accessor
		within the host space which can receive requests.
		</p>
		<xsl:call-template name="proto-instantiate-example"/>
		
		<xsl:call-template name="parameters"/>
		
	</xsl:template>
	
	<xsl:template name="transport">
		<h2>Transports</h2>
		<p>The <xsl:value-of select="name"/> is a <a href="{$resolver}doc:logicalreference:module:standard:logical:transport">transport</a>
		that must be instantiated from a <a href="{$resolver}doc:logicalreference:module:standard:logical:prototype">prototype</a>.
		Transports issue a root request into their hosting space when they detect an external event.</p>
		
		<xsl:call-template name="proto-instantiate-example"/>
		
		<xsl:call-template name="parameters"/>
	</xsl:template>	
	
	<xsl:template name="representation">
		<!-- 
		<h2>Representations</h2>
		<p>to do....</p>
		 -->
	</xsl:template>
	
	<xsl:template name="proto-instantiate-example">
		<p>Here is an example of how to instantiate an instance of <xsl:value-of select="name"/>:
		{xml}
		<overlay>
			<prototype><xsl:value-of select="identifier"/></prototype>
			<xsl:for-each select="parameters/parameter[name!='space' and name!='private' and name!='id']">
				<xsl:element name="{name}">
					<xsl:choose>
						<xsl:when test="desc"><xsl:value-of select="desc"/></xsl:when>
						<xsl:when test="name='private'"/>
						<xsl:when test="name='id'"/>
						<xsl:when test="name='grammar'">an identifier grammar definition</xsl:when>
						<xsl:when test="contains(class,'ISpace')">a nested space</xsl:when>
						<xsl:when test="contains(class,'IIdentifier')">a resource identifier or literal xml fragment</xsl:when>
						<xsl:when test="contains(class,'Integer')">an integer value</xsl:when>
						<xsl:when test="contains(class,'Long')">a long value</xsl:when>
						<xsl:when test="contains(class,'String')">a string value</xsl:when>
						<xsl:when test="contains(class,'Float')">a float value</xsl:when>
						<xsl:when test="contains(class,'Double')">a double value</xsl:when>
						<xsl:when test="contains(class,'Boolean')">a boolean value</xsl:when>
						<xsl:when test="contains(class,'Element')">a literal xml fragment</xsl:when>
					</xsl:choose>
				</xsl:element>
			</xsl:for-each>
			<xsl:if test="parameters/parameter[name='space']">
				<space>
					<xsl:comment>wrapped space...</xsl:comment>
				</space>
			</xsl:if>
		</overlay>
		{/xml}</p>
	</xsl:template>
	
	<xsl:template name="parameters">
		<h2>Parameters</h2>
		<p>Parameters are tags which customise the instantiation of the prototype.
		<xsl:choose>
			<xsl:when test="parameters/parameter[not(contains(class,'ISpace'))]">
			The <xsl:value-of select="name"/> has the following customisable parameters:<p/>
			<table class="border-table">
				<tr class="table-title">
					<td>parameter</td>
					<td>details</td>
				</tr>
				<xsl:for-each select="parameters/parameter[not(contains(class,'ISpace'))]">
					<tr>
						<td><xsl:value-of select="name"/></td>
						<td>
							<xsl:choose>
								<xsl:when test="desc"><div><xsl:value-of select="desc"/></div></xsl:when>
								<xsl:when test="name='grammar'">
									<div>An <a href="{$resolver}doc:logicalreference:module:standard:logical:grammar">identifier grammar</a> definition.</div>
								</xsl:when>
								<xsl:when test="name='id'">
									<div>An identifier for instantiated endpoint, if omitted an auto-generated it will be used.</div>
								</xsl:when>
								<xsl:when test="name='private'">
									<div>If included this parameter will mark the endpoint as private and it will not be exposed outside module.</div>
								</xsl:when>
							</xsl:choose>
							<div>
							The <xsl:value-of select="name"/> parameter 
							<xsl:choose>
								<xsl:when test="min='1' and max='1'">is mandatory</xsl:when>
								<xsl:when test="min='0' and max='1'">is optional</xsl:when>
								<xsl:when test="min='0' and max&gt;'32768'">may be specified any number of times</xsl:when>
								<xsl:otherwise>may be specified between <xsl:value-of select="min"/> and
									<xsl:choose>
										<xsl:when test="max&gt;'32768'">infinity</xsl:when>
										<xsl:otherwise><xsl:value-of select="max"/></xsl:otherwise>
									</xsl:choose>
								</xsl:otherwise>
							</xsl:choose>
							<xsl:text>. It is typed as </xsl:text>
							<xsl:choose>
								<xsl:when test="contains(class,'ISpace')">a nested space</xsl:when>
								<xsl:when test="contains(class,'IIdentifier')">a resource identifier or literal xml fragment</xsl:when>
								<xsl:when test="contains(class,'Integer')">an integer value</xsl:when>
								<xsl:when test="contains(class,'Long')">a long value</xsl:when>
								<xsl:when test="contains(class,'String')">a string value</xsl:when>
								<xsl:when test="contains(class,'Float')">a float value</xsl:when>
								<xsl:when test="contains(class,'Double')">a double value</xsl:when>
								<xsl:when test="contains(class,'Boolean')">a boolean value</xsl:when>
								<xsl:when test="contains(class,'Element')">a literal xml fragment</xsl:when>
							</xsl:choose>
							<xsl:text> and </xsl:text>						
							<xsl:choose>
								<xsl:when test="default='no default'">it has no default value.</xsl:when>
								<xsl:otherwise>its default value is "<xsl:value-of select="default"/>".</xsl:otherwise>
							</xsl:choose>
							</div>
						</td>
					</tr>
				</xsl:for-each>
			</table>
			</xsl:when>
			<xsl:otherwise>The <xsl:value-of select="name"/> has no customisable parameters.</xsl:otherwise>
		</xsl:choose>
		</p>
	</xsl:template>
		
</xsl:stylesheet>
