package org.netkernel.doc.endpoint;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;


/**
 * This engine will create a link to JavaDoc for the provided class.
 *
 * TODO: Distinguish between different families of Java classes and figure out the right documentation source.
 *
 * TODO: How do we have a global resource that relativizes this?
 */


public class JavaDocEngine extends StandardAccessorImpl
  {
  public JavaDocEngine()
    {
    this.declareThreadSafe();
    }


  @Override
  public void onSource(INKFRequestContext aContext) throws Exception
    {
    String type = aContext.getThisRequest().getArgumentValue("activeType");
    String operandValue = aContext.source("arg:operand", String.class);

    boolean classOnly = type.startsWith("wikiTemplateEngine/JAVADOCCLASS");
    /*
      This is a work-in-progress. For now we will detect an increasing number of standard Sun Java
      classes and create links to the Java 6 documentation on the web.

      Later we will add more additional categories

      For those that don't match we don't create a link, just emit plain text.

     */

    String className = operandValue.substring(operandValue.lastIndexOf(".")+1);
    String classAndPath = operandValue;
    String link;
    if (classOnly)
        {
        link = className;
        }
    else
        {
        link = classAndPath;
        }


    if (matchesSunJava6(classAndPath))
        {
        String javadocURL = "http://java.sun.com/javase/6/docs/api/index.html?" + classAndPath.replaceAll("\\.", "/") + ".html";


        if (type.endsWith("XHTML"))
          {
          // XHTML
          aContext.createResponseFrom("<span style=\"\"><a href=\"" + javadocURL + "\" >" + link + "</a></span>");
          }
        else
          {  //Docbook
          aContext.createResponseFrom("<note><![CDATA[" + link + "]]></note>");
          }
        }
    else
        {
        aContext.createResponseFrom(link);
        }

    }


    private boolean matchesSunJava6(String className)
        {
        boolean matches = false;

        if (className.startsWith("java.lang.")) { matches = true;}
        if (className.startsWith("java.awt.")) { matches = true;}
        if (className.startsWith("java.beans.")) { matches = true;}
        if (className.startsWith("java.io.")) { matches = true;}
        if (className.startsWith("java.lang.")) { matches = true;}
        if (className.startsWith("java.math.")) { matches = true;}
        if (className.startsWith("java.net.")) { matches = true;}
        if (className.startsWith("java.nio.")) { matches = true;}
        if (className.startsWith("java.rmi.")) { matches = true;}
        if (className.startsWith("java.security.")) { matches = true;}
        if (className.startsWith("java.sql.")) { matches = true;}
        if (className.startsWith("java.text.")) { matches = true;}
        if (className.startsWith("java.util.")) { matches = true;}
        if (className.startsWith("javax.accessibility.")) { matches = true;}
        if (className.startsWith("javax.activation.")) { matches = true;}
        if (className.startsWith("javax.annotation.")) { matches = true;}
        if (className.startsWith("javax.crypto.")) { matches = true;}
        if (className.startsWith("javax.imageio.")) { matches = true;}
        if (className.startsWith("javax.jws.")) { matches = true;}
        if (className.startsWith("javax.lang.")) { matches = true;}
        if (className.startsWith("javax.management.")) { matches = true;}
        if (className.startsWith("javax.naming.")) { matches = true;}
        if (className.startsWith("javax.net.")) { matches = true;}
        if (className.startsWith("javax.print.")) { matches = true;}
        if (className.startsWith("javax.rmi.")) { matches = true;}
        if (className.startsWith("javax.script.")) { matches = true;}
        if (className.startsWith("javax.security.")) { matches = true;}
        if (className.startsWith("javax.sound.")) { matches = true;}
        if (className.startsWith("javax.sql.")) { matches = true;}
        if (className.startsWith("javax.swing.")) { matches = true;}
        if (className.startsWith("javax.sql.")) { matches = true;}
        if (className.startsWith("javax.tools.")) { matches = true;}
        if (className.startsWith("javax.transaction.")) { matches = true;}
        if (className.startsWith("javax.xml.")) { matches = true;}
        if (className.startsWith("org.w3c.dom.")) { matches = true;}
        if (className.startsWith("org.xml.")) { matches = true;}

        return matches;
        }

  }
